@@ -94,9 +94,11 @@ from __future__ import division, print_function
 # import argparse  # FIX
 # import io        # FIX
 import sys
-import unicodedata
+
+# import unicodedata  # FIX
 import unittest
 from dataclasses import dataclass  # FIX
+from typing import List
 
 # CoNLL-U column names
 ID, FORM, LEMMA, UPOS, XPOS, FEATS, HEAD, DEPREL, DEPS, MISC = range(10)
@@ -181,7 +183,7 @@ def _encode(text):
 
 # Load given CoNLL-U file into internal representation
 # def load_conllu(file):
-def load_conllu(lines: list[str]):  # FIX
+def load_conllu(lines: List[str]):  # FIX
     # Internal representation classes
     class UDRepresentation:
         def __init__(self):
@@ -292,9 +294,10 @@ def load_conllu(lines: list[str]):  # FIX
         # Delete spaces from FORM, so gold.characters == system.characters
         # even if one of them tokenizes the space. Use any Unicode character
         # with category Zs.
-        columns[FORM] = "".join(filter(lambda c: unicodedata.category(c) != "Zs", columns[FORM]))
-        if not columns[FORM]:
-            raise UDError("There is an empty FORM in the CoNLL-U file")
+        # FIX
+        # columns[FORM] = "".join(filter(lambda c: unicodedata.category(c) != "Zs", columns[FORM]))
+        # if not columns[FORM]:
+        #     raise UDError("There is an empty FORM in the CoNLL-U file")
 
         # Save token
         ud.characters.extend(columns[FORM])
@@ -577,7 +580,7 @@ def evaluate(gold_ud, system_ud):
 # def load_conllu_file(path):
 #     _file = open(path, mode="r", **({"encoding": "utf-8"} if sys.version_info >= (3, 0) else {}))
 #     return load_conllu(_file)
-def load_conllu_lines(lines: list[str]):  # FIX
+def load_conllu_lines(lines: List[str]):  # FIX
     return load_conllu(lines)
 
 
@@ -594,8 +597,8 @@ def evaluate_wrapper(args):  # FIX
 
 @dataclass  # FIX
 class Args:
-    gold_lines: list[str]
-    system_lines: list[str]
+    gold_lines: List[str]
+    system_lines: List[str]
     verbose: bool
     counts: bool
 
