@@ -1,4 +1,4 @@
-from typing import Union
+from typing import Dict, Union
 
 import torch
 from torchmetrics import Metric
@@ -8,6 +8,8 @@ from kwja.utils.reading import UNK_ID
 
 
 class ReadingPredictorMetric(Metric):
+    full_state_update = False
+
     def __init__(self) -> None:
         super().__init__()
         self.add_state("predictions", default=[], dist_reduce_fx="cat")
@@ -17,7 +19,7 @@ class ReadingPredictorMetric(Metric):
         self.predictions.append(predictions)
         self.labels.append(labels)
 
-    def compute(self) -> dict[str, Union[torch.Tensor, float]]:
+    def compute(self) -> Dict[str, Union[torch.Tensor, float]]:
         predictions = self.predictions.view(-1)
         labels = self.labels.view(-1)
         ignore_indexes = labels == IGNORE_INDEX
