@@ -1,6 +1,6 @@
 from collections import OrderedDict
 from math import sqrt
-from typing import Optional
+from typing import Dict, List, Optional, Tuple
 
 import torch
 import torch.nn as nn
@@ -59,8 +59,8 @@ class PhraseAnalyzer(nn.Module):
             )
         )
 
-    def forward(self, pooled_outputs: torch.Tensor, batch: dict[str, torch.Tensor]) -> dict[str, torch.Tensor]:
-        outputs: dict[str, torch.Tensor] = {}
+    def forward(self, pooled_outputs: torch.Tensor, batch: Dict[str, torch.Tensor]) -> Dict[str, torch.Tensor]:
+        outputs: Dict[str, torch.Tensor] = {}
         ne_logits, emissions = self.compute_ne_logits(pooled_outputs, batch)
         outputs["ne_logits"] = ne_logits
         word_feature_logits = self.word_feature_head(pooled_outputs)  # (batch_size, seq_length, num_word_features)
@@ -90,8 +90,8 @@ class PhraseAnalyzer(nn.Module):
         return outputs
 
     def compute_ne_logits(
-        self, pooled_outputs: torch.Tensor, batch: dict[str, torch.Tensor]
-    ) -> tuple[torch.Tensor, torch.Tensor]:
+        self, pooled_outputs: torch.Tensor, batch: Dict[str, torch.Tensor]
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         ne_logits = self.ne_head(pooled_outputs)  # (batch_size, seq_length, num_tags)
         device, shape = map(lambda x: getattr(ne_logits, x), ["device", "shape"])
         non_target = torch.full_like(ne_logits, -1024.0)
@@ -143,7 +143,7 @@ class CRF(nn.Module):
     .. _Viterbi algorithm: https://en.wikipedia.org/wiki/Viterbi_algorithm
     """
 
-    def __init__(self, tags: tuple[str, ...], batch_first: bool = True) -> None:
+    def __init__(self, tags: Tuple[str, ...], batch_first: bool = True) -> None:
         num_tags = len(tags)
         assert num_tags > 0, f"invalid number of tags: {num_tags}"
         super().__init__()
@@ -156,7 +156,7 @@ class CRF(nn.Module):
 
         self.reset_parameters(tags)
 
-    def reset_parameters(self, tags: tuple[str, ...]) -> None:
+    def reset_parameters(self, tags: Tuple[str, ...]) -> None:
         bound = sqrt(6 / self.num_tags)
         nn.init.uniform_(self.start_transitions, -bound, bound)
         nn.init.uniform_(self.end_transitions, -bound, bound)
@@ -180,19 +180,19 @@ class CRF(nn.Module):
 
     def forward(
         self,
-        emissions: torch.FloatTensor,
-        tags: torch.LongTensor,
-        mask: Optional[torch.ByteTensor] = None,
+        emissions: torch.Tensor,
+        tags: torch.Tensor,
+        mask: Optional[torch.Tensor] = None,
         reduction: str = "token_mean",
     ) -> torch.Tensor:
         """Compute the conditional log likelihood of a sequence of tags given emission scores.
 
         Args:
-            emissions (`~torch.FloatTensor`): Emission score tensor of size ``(seq_length, batch_size, num_tags)`` or
+            emissions (`~torch.Tensor`): Emission score tensor of size ``(seq_length, batch_size, num_tags)`` or
                                               ``(batch_size, seq_length, num_tags)`` (if batch_first is True).
-            tags (`~torch.LongTensor`): Sequence of tags tensor of size ``(seq_length, batch_size)`` or
+            tags (`~torch.Tensor`): Sequence of tags tensor of size ``(seq_length, batch_size)`` or
                                         ``(batch_size, seq_length, num_tags)`` (if batch_first is True).
-            mask (`~torch.ByteTensor`): Mask tensor of size ``(seq_length, batch_size)`` or
+            mask (`~torch.Tensor`): Mask tensor of size ``(seq_length, batch_size)`` or
                                         ``(batch_size, seq_length)`` (if batch_first is True).
             reduction: Specifies  the reduction to apply to the output: ``none|sum|mean|token_mean``.
                        ``none``: no reduction will be applied.
@@ -208,7 +208,7 @@ class CRF(nn.Module):
         if reduction not in ("none", "sum", "mean", "token_mean"):
             raise ValueError(f"invalid reduction: {reduction}")
         if mask is None:
-            mask = torch.ones_like(tags, dtype=torch.uint8)
+            mask = torch.ones_like(tags, dtype=torch.bool)
 
         if self.batch_first:
             emissions = emissions.transpose(0, 1)
@@ -228,7 +228,7 @@ class CRF(nn.Module):
         assert reduction == "token_mean"
         return llh.sum() / mask.type_as(emissions).sum()
 
-    def decode(self, emissions: torch.FloatTensor, mask: Optional[torch.ByteTensor] = None) -> list[list[int]]:
+    def decode(self, emissions: torch.Tensor, mask: Optional[torch.Tensor] = None) -> List[List[int]]:
         """Find the most likely tag sequence using Viterbi algorithm.
 
         Args:
@@ -241,20 +241,19 @@ class CRF(nn.Module):
             `~list[list[int]]`: list of list containing the best tag sequence for each batch.
         """
         self._validate(emissions, mask=mask)
-        if mask is None:
-            mask = emissions.new_ones(emissions.shape[:2], dtype=torch.uint8)
+        mask_: torch.Tensor = mask or emissions.new_ones(emissions.shape[:2], dtype=torch.bool)
 
         if self.batch_first:
             emissions = emissions.transpose(0, 1)
-            mask = mask.transpose(0, 1)
+            mask_ = mask_.transpose(0, 1)
 
-        return self._viterbi_decode(emissions, mask)
+        return self._viterbi_decode(emissions, mask_)
 
     def _validate(
         self,
-        emissions: torch.FloatTensor,
-        tags: Optional[torch.LongTensor] = None,
-        mask: Optional[torch.ByteTensor] = None,
+        emissions: torch.Tensor,
+        tags: Optional[torch.Tensor] = None,
+        mask: Optional[torch.Tensor] = None,
     ) -> None:
         if emissions.dim() != 3:
             raise ValueError(f"emissions must have dimension of 3, got {emissions.dim()}")
@@ -279,9 +278,7 @@ class CRF(nn.Module):
             # if not no_empty_seq and not no_empty_seq_bf:
             #     raise ValueError('mask of the first timestep must all be on')
 
-    def _compute_score(
-        self, emissions: torch.FloatTensor, tags: torch.LongTensor, mask: torch.ByteTensor
-    ) -> torch.Tensor:
+    def _compute_score(self, emissions: torch.Tensor, tags: torch.Tensor, mask: torch.Tensor) -> torch.Tensor:
         # emissions: (seq_length, batch_size, num_tags)
         # tags: (seq_length, batch_size)
         assert emissions.dim() == 3 and tags.dim() == 2
@@ -314,7 +311,7 @@ class CRF(nn.Module):
 
         return score
 
-    def _compute_normalizer(self, emissions: torch.FloatTensor, mask: torch.ByteTensor) -> torch.Tensor:
+    def _compute_normalizer(self, emissions: torch.Tensor, mask: torch.Tensor) -> torch.Tensor:
         # emissions: (seq_length, batch_size, num_tags)
         # mask: (seq_length, batch_size)
         assert emissions.dim() == 3 and mask.dim() == 2
@@ -353,7 +350,7 @@ class CRF(nn.Module):
         # Sum (log-sum-exp) over all possible tags
         return torch.logsumexp(score, dim=1)  # (batch_size,)
 
-    def _viterbi_decode(self, emissions: torch.FloatTensor, mask: torch.ByteTensor) -> list[list[int]]:
+    def _viterbi_decode(self, emissions: torch.Tensor, mask: torch.Tensor) -> List[List[int]]:
         # emissions: (seq_length, batch_size, num_tags)
         # mask: (seq_length, batch_size)
         assert emissions.dim() == 3 and mask.dim() == 2
@@ -398,7 +395,7 @@ class CRF(nn.Module):
 
         # Now, compute the best path for each sample
 
-        seq_ends = mask.long().sum(dim=0) - 1  # (batch_size,)
+        seq_ends: torch.Tensor = mask.long().sum(dim=0) - 1  # (batch_size,)
         best_tags_list = []
 
         for idx in range(batch_size):
@@ -408,7 +405,9 @@ class CRF(nn.Module):
 
             # We trace back where the best last tag comes from, append that to our best tag sequence, and
             # trace it back again, and so on
-            for hist in reversed(history[: seq_ends[idx]]):
+            end_index = seq_ends[idx].item()
+            assert isinstance(end_index, int)
+            for hist in reversed(history[:end_index]):
                 best_last_tag = hist[idx][best_tags[-1]]
                 best_tags.append(best_last_tag.item())
 
