@@ -1,9 +1,10 @@
 from collections import defaultdict
+from typing import Dict, Set
 
 
 class DependencyManager:
     def __init__(self) -> None:
-        self.directed_graph: dict[int, set] = defaultdict(set)
+        self.directed_graph: Dict[int, set] = defaultdict(set)
         self.root = False
 
     def add_edge(self, src: int, dst: int) -> None:
@@ -12,7 +13,7 @@ class DependencyManager:
     def remove_edge(self, src: int, dst: int) -> None:
         self.directed_graph[src].remove(dst)
 
-    def is_cyclic(self, src: int, visited: set[int], cache: dict[int, bool]) -> bool:
+    def is_cyclic(self, src: int, visited: Set[int], cache: Dict[int, bool]) -> bool:
         if src in cache:
             return cache[src]
 
@@ -31,8 +32,8 @@ class DependencyManager:
         return ret
 
     def has_cycle(self) -> bool:
-        visited: set[int] = set()
-        cache: dict[int, bool] = {}
+        visited: Set[int] = set()
+        cache: Dict[int, bool] = {}
         for src in list(self.directed_graph.keys()):
             if self.is_cyclic(src, visited, cache):
                 return True
