@@ -1,9 +1,10 @@
-from typing import Any, MutableMapping, MutableSequence, Union
+from omegaconf import DictConfig, ListConfig
 
 
 def filter_dict_items(
-    item: MutableMapping[str, Any], keys_to_ignore: MutableSequence[Union[str, MutableMapping[str, MutableSequence]]]
-) -> MutableMapping[str, Any]:
+    item: DictConfig,
+    keys_to_ignore: ListConfig,
+) -> DictConfig:
     """Filter out dictionary items whose key is in keys_to_ignore recursively."""
     for key, value in item.items():
         ignore = False
@@ -11,7 +12,7 @@ def filter_dict_items(
             if isinstance(key_to_ignore, str) and key == key_to_ignore:
                 ignore = True
                 break
-            elif isinstance(key_to_ignore, MutableMapping) and key in key_to_ignore:
+            elif isinstance(key_to_ignore, (dict, DictConfig)) and key in key_to_ignore:
                 item[key] = filter_dict_items(value, key_to_ignore[key])
                 break
         if ignore is True:
